package com.engine.jclproduct.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.cloudstore.eccom.result.WeaResultMsg;
import com.engine.core.impl.Service;
import com.engine.jclproduct.component.WeaverWeaTable;
import com.engine.jclproduct.entity.param.TransferParam;
import com.engine.jclproduct.entity.vo.*;
import com.engine.jclproduct.service.EmployeeRecordReportService;
import com.engine.jclproduct.utils.EmploymentUtil;
import com.engine.jclproduct.utils.ExcelUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.hrm.company.DepartmentComInfo;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Author liang.cheng
 * @Date 2024/10/25 11:11 AM
 * @Description: TODO
 * @Version 1.0
 */
public class EmployeeRecordReportServiceImpl extends Service implements EmployeeRecordReportService {

    @Override
    public Map<String, Object> commonReport(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>();
        String selectedKey = Util.null2String(params.get("selectedKey"));
        switch (selectedKey) {
            case "0":
                result = timeResourece(params);
                break;
            case "1":
                result = leaveResourece(params);
                break;
            case "2":
                result = transferResourece(params);
                break;
            case "3":
                result = caluResourece(params);
                break;
            default:
                break;
        }


        return result;
    }

    @Override
    public Map<String, Object> timeResourece(Map<String, Object> params) {
        Map<String, Object> resultMap = new HashMap<>(8);
        
        WeaverWeaTable<TimeResourceVo> table = new WeaverWeaTable<>(user, TimeResourceVo.class);
        String sqlWhere = buildtimeResoureceSql(params);
        table.setSqlwhere(sqlWhere);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        resultMap.putAll(result.getResultMap());
        return resultMap;
    }

    @Override
    public Map<String, Object> leaveResourece(Map<String, Object> params) {
        Map<String, Object> resultMap = new HashMap<>(8);

        WeaverWeaTable<TimeResourceVo> table = new WeaverWeaTable<>(user, LeaveResourceVo.class);
        String sqlWhere = buildLeaveResoureceSql(params);
        sqlWhere += " and employeeStatus in (5)";
        table.setSqlwhere(sqlWhere);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        resultMap.putAll(result.getResultMap());
        return resultMap;
    }

    @Override
    public Map<String, Object> transferResourece(Map<String, Object> params) {
        Map<String, Object> resultMap = new HashMap<>(8);

        WeaverWeaTable<TimeResourceVo> table = new WeaverWeaTable<>(user, TransferResourceVo.class);
        String sqlWhere = buildtransferResoureceSql(params);
        sqlWhere += " and changType in (2)";
        table.setSqlwhere(sqlWhere);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        resultMap.putAll(result.getResultMap());
        return resultMap;
    }

    @Override
    public Map<String, Object> caluResourece(Map<String, Object> params) {
        RecordSet rs = new RecordSet();
        Map<String, Object> resultMap = new HashMap<>(4);

        List<CaluResourceVo> caluResourceVos = new ArrayList<>();
        String departmentId = Util.null2String(params.get("departmentId"));
        String date = Util.null2String(params.get("date"));
        DepartmentComInfo departmentComInfo = new DepartmentComInfo();

        String sql = "select id,supdepid,departmentname from hrmdepartment where (canceled is null or canceled != 1) ";
        if(!"".equals(departmentId)){
            ArrayList<String> lists = new ArrayList<>();
            lists.add(departmentId);
            departmentComInfo.getAllChildDeptByDepId(lists,departmentId);
            String join = CollectionUtil.join(lists, ",");
            sql += " and id in ("+join+")";
        }

        sql += " order by showorder,id asc";

        //1.获取部门数据信息
        rs.executeQuery(sql);
        while (rs.next()) {
            caluResourceVos.add(CaluResourceVo.builder()
                    .key(Util.getIntValue(rs.getString("id")))
                    .superKey(Util.getIntValue(rs.getString("supdepid"),0))
                    .departmentName(Util.null2String(rs.getString("departmentname")))
            .build());
        }

        //2.人数统计计算
        String firstDayOfMonth = EmploymentUtil.firstDayOfMonth(date);
        String lastDayOfMonth = EmploymentUtil.lastDayOfMonth(date);
        //期初人数
        Map<Integer, Integer> startMap = EmploymentUtil.sumPerson(firstDayOfMonth, "0,1,2,3");
        //期末人数
        Map<Integer, Integer> endMap = EmploymentUtil.sumPerson(lastDayOfMonth, "0,1,2,3");
        //入职人数
        Map<Integer, Integer> entryMap = EmploymentUtil.rangePerson(firstDayOfMonth, lastDayOfMonth, "0");
        //转正人数
        Map<Integer, Integer> transferMap = EmploymentUtil.rangePerson(firstDayOfMonth, lastDayOfMonth, "1");
        //离职人数
        Map<Integer, Integer> leaveMap = EmploymentUtil.rangePerson(firstDayOfMonth, lastDayOfMonth, "4,5");
        //调动信息
        List<TransferParam> transferList = EmploymentUtil.transferPerson(firstDayOfMonth, lastDayOfMonth);



        caluResourceVos.forEach(e-> {
            e.setStartNumber(startMap.getOrDefault(e.getKey(), 0));
            e.setEntryNumber(entryMap.getOrDefault(e.getKey(), 0));
            e.setTransferNumber(transferMap.getOrDefault(e.getKey(), 0));
            e.setLeaveNumber(leaveMap.getOrDefault(e.getKey(), 0));
            int oldCount = (int) transferList.stream()
                    .filter(param -> param.getChangeOldDept() != null && param.getChangeOldDept().equals(e.getKey()))
                    .count();
            int newCount = (int) transferList.stream()
                    .filter(param -> param.getChangeNewDept() != null && param.getChangeNewDept().equals(e.getKey()))
                    .count();
            e.setOnNumber(newCount);
            e.setOutNumber(oldCount);
            e.setEndNumber(endMap.getOrDefault(e.getKey(), 0));
        });


        resultMap.put("caluResourceVos",caluResourceVos);
        //3.构建上下级关系
        List<CaluResourceVo> topLevelResources = new ArrayList<>();

        Map<Integer, CaluResourceVo> resourceMap = caluResourceVos.stream()
                .collect(Collectors.toMap(CaluResourceVo::getKey, Function.identity()));

        caluResourceVos.forEach(resource -> {
            if (resource.getSuperKey() == 0 || String.valueOf(resource.getKey()).equals(departmentId)) {
                // 如果 superKey 为 0，说明是顶级元素 或者departmentId出现
                topLevelResources.add(resource);
            } else {
                CaluResourceVo parent = resourceMap.get(resource.getSuperKey());
                if (parent != null) {
                    if (parent.getChildren() == null) {
                        parent.setChildren( new ArrayList<>());
                    }
                    parent.getChildren().add(resource);
                }
            }
        });

        resultMap.put("dataSource",topLevelResources);
        return resultMap;
    }

    @Override
    public XSSFWorkbook reportExport(Map<String, Object> params) {
        String sheetName = "管理幅度统计";
        Map<String, Object> map = caluResourece(params);
        List<CaluResourceVo> caluResourceVos =(List<CaluResourceVo>) map.get("caluResourceVos");
        List<Object> headerList = Arrays.asList("部门名称","期初人数","入职人数","转正人数","离职人数","调入人数","调出人数","期末人数");
        List<List<Object>> rows = new LinkedList<>();
        for (CaluResourceVo hr : caluResourceVos) {
            List<Object> row = new LinkedList<>();
            row.add(hr.getDepartmentName());
            row.add(hr.getStartNumber());
            row.add(hr.getEntryNumber());
            row.add(hr.getTransferNumber());
            row.add(hr.getLeaveNumber());
            row.add(hr.getOnNumber());
            row.add(hr.getOutNumber());
            row.add(hr.getEndNumber());
            rows.add(row);
        }
        List<List<Object>> excelSheetData = new ArrayList<>();
        excelSheetData.add(headerList);
        excelSheetData.addAll(rows);
        return ExcelUtil.genWorkbookV2(excelSheetData, sheetName);
    }

    @Override
    public Map<String, Object> orgReport(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>(8);
        String selectedKey = Util.null2String(params.get("selectedKey"));
        switch (selectedKey) {
            case "0":
                result = timeOrganization(params);
                break;
            case "1":
                result = transOrganization(params);
                break;
            default:
                break;
        }
        return result;
    }

    private Map<String, Object> transOrganization(Map<String, Object> params) {
        Map<String, Object> resultMap = new HashMap<>(8);
        String type = Util.null2String(params.get("type"));
        WeaResultMsg result = new WeaResultMsg(false);
        if ("0".equals(type)) {
            WeaverWeaTable<TransSubCompanyVo> table = new WeaverWeaTable<>(user, TransSubCompanyVo.class);
            String sqlWhere = buildtransOrgSql(params);
            table.setSqlwhere(sqlWhere);
            result.putAll(table.makeDataResult());
        }else {
            WeaverWeaTable<TransDepartmentVo> table = new WeaverWeaTable<>(user, TransDepartmentVo.class);
            String sqlWhere = buildtransOrgSql(params);
            table.setSqlwhere(sqlWhere);
            result.putAll(table.makeDataResult());
        }
        result.success();
        resultMap.putAll(result.getResultMap());

        return resultMap;
    }

    private String buildtransOrgSql(Map<String, Object> params) {
        String pointValue = Util.null2String(params.get("pointValue"));
        String startDate = Util.null2String(params.get("startDate"));
        String endDate = Util.null2String(params.get("endDate"));
        String type = Util.null2String(params.get("type"));

        String sql = getLeaveSql(pointValue,startDate,endDate);
        String sqlWhere = " where  changeType = 1 and oType = "+type+sql;
        String statusValue = Util.null2String(params.get("statusValue"));
        if (StringUtils.isNotBlank(statusValue)) {
            sqlWhere += " and orgStatus in ("+statusValue+")";
        }
        String subCompanyId = Util.null2String(params.get("subCompanyId"));
        if (StringUtils.isNotBlank(subCompanyId)) {
            if("0".equals(type)) {
                sqlWhere += " and oId ="+subCompanyId;
            }else {
                sqlWhere += " and belongSub ="+subCompanyId;
            }
        }
        String departmentId = Util.null2String(params.get("departmentId"));
        if (StringUtils.isNotBlank(departmentId) && "1".equals(type)) {
            sqlWhere += " and oId ="+departmentId;
        }

        return sqlWhere;

    }

    private Map<String, Object> timeOrganization(Map<String, Object> params) {
        Map<String, Object> resultMap = new HashMap<>(8);
        String type = Util.null2String(params.get("type"));
        WeaResultMsg result = new WeaResultMsg(false);
        if ("0".equals(type)) {
            WeaverWeaTable<TimeSubCompanyVo> table = new WeaverWeaTable<>(user, TimeSubCompanyVo.class);
            String sqlWhere = buildtimeOrgSql(params);
            table.setSqlwhere(sqlWhere);
            result.putAll(table.makeDataResult());
        }else {
            WeaverWeaTable<TimeDepartmentVo> table = new WeaverWeaTable<>(user, TimeDepartmentVo.class);
            String sqlWhere = buildtimeOrgSql(params);
            table.setSqlwhere(sqlWhere);
            result.putAll(table.makeDataResult());
        }
        result.success();
        resultMap.putAll(result.getResultMap());

        return resultMap;
    }

    private String buildtimeOrgSql(Map<String, Object> params) {
        String pointValue = Util.null2String(params.get("pointValue"));
        String startDate = Util.null2String(params.get("startDate"));
        String type = Util.null2String(params.get("type"));

        String sql = getPointSql(pointValue,startDate);
        String sqlWhere = " where oType = "+type+" and "+sql;

        String statusValue = Util.null2String(params.get("statusValue"));
        if (StringUtils.isNotBlank(statusValue)) {
            sqlWhere += " and orgStatus in ("+statusValue+")";
        }
        String subCompanyId = Util.null2String(params.get("subCompanyId"));
        if (StringUtils.isNotBlank(subCompanyId)) {
            if("0".equals(type)) {
                sqlWhere += " and oId ="+subCompanyId;
            }else {
                sqlWhere += " and belongSub ="+subCompanyId;
            }
        }
        String departmentId = Util.null2String(params.get("departmentId"));
        if (StringUtils.isNotBlank(departmentId) && "1".equals(type)) {
            sqlWhere += " and oId ="+departmentId;
        }

        return sqlWhere;
    }

    private String buildtransferResoureceSql(Map<String, Object> params) {
        String pointValue = Util.null2String(params.get("pointValue"));
        String startDate = Util.null2String(params.get("startDate"));
        String endDate = Util.null2String(params.get("endDate"));
        String sql = getLeaveSql(pointValue,startDate,endDate);
        String sqlWhere = " where 1=1 "+sql;
        String resourceId = Util.null2String(params.get("resourceId"));
        if (StringUtils.isNotBlank(resourceId)) {
            sqlWhere += " and userId ="+resourceId;
        }

        String departmentId = Util.null2String(params.get("departmentId"));
        if (StringUtils.isNotBlank(departmentId)) {
            sqlWhere += " and departmentId ="+departmentId;
        }

        String statusValue = Util.null2String(params.get("statusValue"));
        if (StringUtils.isNotBlank(statusValue)) {
            sqlWhere += " and employeeStatus in ("+statusValue+")";
        }

        return sqlWhere;
    }


    private String buildLeaveResoureceSql(Map<String, Object> params) {
        String pointValue = Util.null2String(params.get("pointValue"));
        String startDate = Util.null2String(params.get("startDate"));
        String endDate = Util.null2String(params.get("endDate"));
        String sql = getLeaveSql(pointValue,startDate,endDate);
        String sqlWhere = " where 1=1 "+sql;
        String departmentId = Util.null2String(params.get("departmentId"));
        if (StringUtils.isNotBlank(departmentId)) {
            sqlWhere += " and departmentId ="+departmentId;
        }

        return sqlWhere;
    }

    private String getLeaveSql(String pointValue, String startDate, String endDate) {
        String sql = "";
        switch (pointValue){
            case "0":
                break;
            case "1":
                String today = DateUtil.getCurrentDate();
                sql += " and startDate = '"+today+"' and stopDate >= '"+today+"'";
                break;
            case "2":
                String firstDayOfWeek = DateUtil.getFirstDayOfWeek();
                String lastDayOfWeek = DateUtil.getLastDayOfWeek();
                sql += " and startDate >= '"+firstDayOfWeek+"' and startDate <= '"+lastDayOfWeek+"' and stopDate >= '"+lastDayOfWeek+"'";
                break;
            case "3":
                String firstDayOfMonth = DateUtil.getFirstDayOfMonthToString();
                String lastDayOfMonth = DateUtil.getLastDayOfMonthToString();
                sql += " and startDate >= '"+firstDayOfMonth+"' and startDate <= '"+lastDayOfMonth+"' and stopDate >= '"+lastDayOfMonth+"'";
                break;
            case "5":
                String firstDayOfYear = DateUtil.getFirstDayOfYear();
                String lastDayOfYear = DateUtil.getLastDayOfYear();
                sql += " and startDate >= '"+firstDayOfYear+"' and startDate <= '"+lastDayOfYear+"'  and stopDate >= '"+lastDayOfYear+"'";
                break;
            case "7":
                String firstDayOfPreviousMonth = EmploymentUtil.firstDayOfPreviousMonth();
                String lastDayOfPreviousMonth = EmploymentUtil.lastDayOfPreviousMonth();
                sql += " and startDate >= '"+firstDayOfPreviousMonth+"' and startDate <= '"+lastDayOfPreviousMonth+"'  and stopDate >= '"+lastDayOfPreviousMonth+"'";
                break;
            case "8":
                String firstDayOfPreviousYear = EmploymentUtil.firstDayOfPreviousYear();
                String lastDayOfPreviousYear = EmploymentUtil.lastDayOfPreviousYear();
                sql += " and startDate >= '"+firstDayOfPreviousYear+"' and startDate <= '"+lastDayOfPreviousYear+"' and stopDate >= '"+lastDayOfPreviousYear+"'";
                break;
            case "6":
                sql += " and startDate >= '"+startDate+"' and startDate <= '"+endDate+"' and stopDate >= '"+endDate+"'";
                break;
            default:
                break;
        }

        return sql;
    }

    private String buildtimeResoureceSql(Map<String, Object> params) {
        String pointValue = Util.null2String(params.get("pointValue"));
        String startDate = Util.null2String(params.get("startDate"));

        String sql = getPointSql(pointValue,startDate);
        String sqlWhere = " where 1=1 and "+sql;

        String statusValue = Util.null2String(params.get("statusValue"));
        if (StringUtils.isNotBlank(statusValue)) {
            sqlWhere += " and employeeStatus in ("+statusValue+")";
        }

        String resourceId = Util.null2String(params.get("resourceId"));
        if (StringUtils.isNotBlank(resourceId)) {
            sqlWhere += " and userId ="+resourceId;
        }

        String departmentId = Util.null2String(params.get("departmentId"));
        if (StringUtils.isNotBlank(departmentId)) {
            sqlWhere += " and departmentId ="+departmentId;
        }

        return sqlWhere;
    }

    private String getPointSql(String pointValue, String startDate) {
        String sql = "";
        switch (pointValue){
            case "0":
                String currentDate = DateUtil.getCurrentDate();
                sql += " startDate <= '"+currentDate+"' and stopDate >= '"+currentDate+"'";
                break;
            case "1":
                String yesterday = DateUtil.getYesterday();
                sql += " startDate <= '"+yesterday+"' and stopDate >= '"+yesterday+"'";
                break;
            case "2":
                String firstDayOfWeek = DateUtil.getFirstDayOfWeek();
                sql += " startDate <= '"+firstDayOfWeek+"' and stopDate >= '"+firstDayOfWeek+"'";
                break;
            case "3":
                String lastDayOfWeek = DateUtil.getLastDayOfWeek();
                sql += " startDate <= '"+lastDayOfWeek+"' and stopDate >= '"+lastDayOfWeek+"'";
                break;
            case "4":
                String firstDayOfMonth = DateUtil.getFirstDayOfMonthToString();
                sql += " startDate <= '"+firstDayOfMonth+"' and stopDate >= '"+firstDayOfMonth+"'";
                break;
            case "5":
                String lastDayOfMonth = DateUtil.getLastDayOfMonthToString();
                sql += " startDate <= '"+lastDayOfMonth+"' and stopDate >= '"+lastDayOfMonth+"'";
                break;
            case "7":
                String firstDayOfYear = DateUtil.getFirstDayOfYear();
                sql += " startDate <= '"+firstDayOfYear+"' and stopDate >= '"+firstDayOfYear+"'";
                break;
            case "8":
                String lastDayOfYear = DateUtil.getLastDayOfYear();
                sql += " startDate <= '"+lastDayOfYear+"' and stopDate >= '"+lastDayOfYear+"'";
                break;
            case "9":
                String firstDayOfPreviousMonth = EmploymentUtil.firstDayOfPreviousMonth();
                sql += " startDate <= '"+firstDayOfPreviousMonth+"' and stopDate >= '"+firstDayOfPreviousMonth+"'";
                break;
            case "10":
                String lastDayOfPreviousMonth = EmploymentUtil.lastDayOfPreviousMonth();
                sql += " startDate <= '"+lastDayOfPreviousMonth+"' and stopDate >= '"+lastDayOfPreviousMonth+"'";
                break;
            case "11":
                String firstDayOfPreviousYear = EmploymentUtil.firstDayOfPreviousYear();
                sql += " startDate <= '"+firstDayOfPreviousYear+"' and stopDate >= '"+firstDayOfPreviousYear+"'";
                break;
            case "12":
                String lastDayOfPreviousYear = EmploymentUtil.lastDayOfPreviousYear();
                sql += " startDate <= '"+lastDayOfPreviousYear+"' and stopDate >= '"+lastDayOfPreviousYear+"'";
                break;
            case "6":
                sql += " startDate <= '"+startDate+"' and stopDate >= '"+startDate+"'";
                break;
            default:
                break;
        }
        return sql;

    }


}
