package com.engine.jclproduct.utils;

import com.engine.jclproduct.entity.param.TransferParam;
import com.engine.jclproduct.entity.po.DepartmentPo;
import com.engine.jclproduct.entity.po.EmploymentRecordPo;
import com.engine.jclproduct.entity.po.HrmresourcePo;
import com.engine.jclproduct.entity.po.SubCompanyPo;
import com.weaver.general.Util;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;

import java.time.LocalDate;
import java.time.Year;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author liang.cheng
 * @Date 2024/10/23 10:14 AM
 * @Description: TODO
 * @Version 1.0
 */
public class EmploymentUtil {


    /**
     * 变动类型
     * @return
     */
    public static  Integer getChangeType(Integer status){

        Integer changeType = null;

        switch (status){
            case 0: case 2: case 3:
                changeType = 0;
                break;
            case 1:
                changeType = 1;
                break;
            case 4: case 5: case 6: case 7:
                changeType = 3;
                break;
            default:
                break;
        }
        return changeType;
    }


    /**
     * 年月日
     * @param companysDate
     * @return
     */
    public static Map<String,Integer> dateSplitExample(String companysDate){

        if ("".equals(companysDate)) {
            companysDate = DateUtil.getCurrentDate();
        }
        LocalDate date = LocalDate.parse(companysDate);
        return new HashMap<String,Integer>(4){
            {
                put("year",date.getYear());
                put("month",date.getMonthValue());
                put("day",date.getDayOfMonth());
            }
        };
    }

    /**
     * 获取当前人员表人员信息
     * @return
     */
    public static List<HrmresourcePo> getHrmResources(){
        List<HrmresourcePo> hrmresourcePoList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select id,workcode,status,companystartdate,companyworkyear,sex,birthday,jobtitle,departmentid,subcompanyid1,managerid from hrmresource order by id asc");
        while (rs.next()) {
            hrmresourcePoList.add(HrmresourcePo.builder()
                    .userId(Util.getIntValue(rs.getString("id")))
                    .workcode(Util.null2String(rs.getString("workcode")))
                    .status(Util.getIntValue(rs.getString("status")))
                    .companyStartDate(Util.null2String(rs.getString("companystartdate")))
                    .companyWorkyear("".equals(Util.null2String(rs.getString("companyworkyear"))) ? "0.00" : Util.null2String(rs.getString("companyworkyear")))
                    .sex(Util.getIntValue(rs.getString("sex")))
                    .birthday(Util.null2String(rs.getString("birthday")))
                    .jobTitle(Util.getIntValue(rs.getString("jobtitle")))
                    .departmentId(Util.getIntValue(rs.getString("departmentid")))
                    .subcompanyId(Util.getIntValue(rs.getString("subcompanyid1")))
                    .managerId(Util.getIntValue(rs.getString("managerid")))
                    .build());
        }

        return hrmresourcePoList;
    }

    /**
     * 当前分部表信息
     * @return
     */
    public static List<SubCompanyPo> getSubCompany() {
        List<SubCompanyPo> subCompanyPoList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select id,subcompanyname,companyid,supsubcomid,showorder,canceled,subcompanycode from hrmsubcompany order by showorder asc");
        while (rs.next()) {
            subCompanyPoList.add(SubCompanyPo.builder()
                    .subcompanyId(Util.getIntValue(rs.getString("id")))
                    .subcompanyName(Util.null2String(rs.getString("subcompanyname")))
                    .companyId(Util.getIntValue(rs.getString("companyid")))
                    .supsubcomId(Util.getIntValue(rs.getString("supsubcomid")))
                    .showorder(Util.getFloatValue(rs.getString("showorder")))
                    .canceled(Util.getIntValue(rs.getString("canceled")))
                    .subcompanyCode(Util.null2String(rs.getString("subcompanycode")))
            .build());
        }
        return subCompanyPoList;
    }


    /**
     * 当前部门表信息
     * @return
     */
    public static List<DepartmentPo> getDepartment() {
        List<DepartmentPo> departmentPoList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select a.id,departmentmark,subcompanyid1,supdepid,showorder,canceled,departmentcode,b.bmfzr from hrmdepartment a\n" +
                " left join hrmdepartmentdefined b on a.id = b.deptid order by showorder asc");
        while (rs.next()) {
            departmentPoList.add(DepartmentPo.builder()
                    .departmentId(Util.getIntValue(rs.getString("id")))
                    .departmentmark(Util.null2String(rs.getString("departmentmark")))
                    .subcompanyId(Util.getIntValue(rs.getString("subcompanyid1")))
                    .supdepId(Util.getIntValue(rs.getString("supdepid")))
                    .showorder(Util.getFloatValue(rs.getString("showorder")))
                    .canceled(Util.getIntValue(rs.getString("canceled")))
                    .departmentCode(Util.null2String(rs.getString("departmentcode")))
                    .bmfzr(Util.null2String(rs.getString("bmfzr")))
                    .build());
        }
        return departmentPoList;
    }




    /**
     * 任职记录 各种状态下 相关变动日期记录
     * @param recordPo
     */
    public static void dutyStatusDate(EmploymentRecordPo recordPo) {
        String currentDate = DateUtil.getCurrentDate();
        switch (recordPo.getEmployeeStatus()){
            case 0:
                recordPo.setProbationStartDate(currentDate);
                break;
            case 1:
                recordPo.setRegularizationDate(currentDate);
                break;
            case 4: case 5:
                recordPo.setLastWorkDate(currentDate);
                break;
            case 7:
                recordPo.setExpectedLastWorkDate(currentDate);
                break;
            default:
                break;
        }

    }

    /**
     * 日期字符串 与当前日期比较
     * @param dateStr1
     * @return
     */
    public static boolean compareDate(String dateStr1){
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        boolean result = false;
        try {
            LocalDate inputDate = LocalDate.parse(dateStr1, formatter);

            LocalDate today = LocalDate.now();

            // 判断当前日期是否大于传入的日期
            if (today.isAfter(inputDate)) {
                result = true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }


    public static String firstDayOfPreviousMonth() {
        LocalDate now = LocalDate.now();
        YearMonth previousMonth = YearMonth.from(now).minusMonths(1);
        LocalDate firstDayOfPreviousMonth = previousMonth.atDay(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
       return  firstDayOfPreviousMonth.format(formatter);
    }


    public static String lastDayOfPreviousMonth() {
        LocalDate now = LocalDate.now();
        YearMonth previousMonth = YearMonth.from(now).minusMonths(1);
        LocalDate lastDayOfPreviousMonth = previousMonth.atEndOfMonth();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        return lastDayOfPreviousMonth.format(formatter);
    }


    public static String firstDayOfPreviousYear() {
        LocalDate now = LocalDate.now();
        Year previousYear = Year.from(now).minusYears(1);
        LocalDate firstDayOfPreviousYear = previousYear.atDay(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        return firstDayOfPreviousYear.format(formatter);
    }

    public static String lastDayOfPreviousYear() {
        LocalDate now = LocalDate.now();
        Year previousYear = Year.from(now).minusYears(1);
        LocalDate lastDayOfPreviousYear = previousYear.atMonth(12).atEndOfMonth();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        return lastDayOfPreviousYear.format(formatter);

    }


    public static String firstDayOfMonth(String yearMonthStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
        YearMonth yearMonth = YearMonth.parse(yearMonthStr, formatter);
        LocalDate firstDayOfMonth = yearMonth.atDay(1);
        return firstDayOfMonth.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));


    }


    public static String lastDayOfMonth(String yearMonthStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
        YearMonth yearMonth = YearMonth.parse(yearMonthStr, formatter);
        LocalDate lastDayOfMonth = yearMonth.atEndOfMonth();
        return lastDayOfMonth.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));

    }


    public static Map<Integer,Integer> sumPerson(String date,String employeeStatus) {
        Map<Integer,Integer> map = new HashMap<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select count(1) as nums,departmentid" +
                " from uf_EmploymentRecord where employeeStatus in ("+employeeStatus+") " +
                " and startDate <= '"+date+"' and stopDate >= '"+date+"'" +
                " group by departmentid");
        while (rs.next()) {
            int departmentid = Util.getIntValue(rs.getString("departmentid"));
            int nums = Util.getIntValue(rs.getString("nums"));
            map.put(departmentid,nums);
        }
        return map;
    }


    public static Map<Integer,Integer> rangePerson(String startDate,String endDate,String employeeStatus) {
        Map<Integer,Integer> map = new HashMap<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select count(1) as nums,departmentid" +
                " from uf_EmploymentRecord where employeeStatus in ("+employeeStatus+") and changtype != 2" +
                " and startDate >= '"+startDate+"' and startDate <= '"+endDate+"' and stopDate >= '"+endDate+"'" +
                " group by departmentid");
        while (rs.next()) {
            int departmentid = Util.getIntValue(rs.getString("departmentid"));
            int nums = Util.getIntValue(rs.getString("nums"));
            map.put(departmentid,nums);
        }
        return map;
    }


    public static List<TransferParam> transferPerson(String startDate,String endDate) {
        List<TransferParam> paramList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        rs.executeQuery("select changeOldDept,changeNewDept " +
                " from uf_EmploymentRecord where changtype = 2 " +
                " and startDate >= '"+startDate+"' and startDate <= '"+endDate+"' and stopDate >= '"+endDate+"'");
        while (rs.next()) {
            int changeOldDept = Util.getIntValue(rs.getString("changeOldDept"));
            int changeNewDept = Util.getIntValue(rs.getString("changeNewDept"));
            paramList.add(TransferParam.builder().changeNewDept(changeNewDept).changeOldDept(changeOldDept).build());
        }
        return paramList;
    }




}
