package com.engine.wysecond.service.impl;

import com.engine.core.impl.Service;
import com.engine.wysecond.entity.*;
import com.engine.wysecond.service.PostionMatrixService;
import com.weaver.general.Util;
import lombok.SneakyThrows;
import weaver.conn.RecordSet;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.resource.ResourceComInfo;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author liang.cheng
 * @Date 2025/2/26 10:04
 * @Description:
 * @Version 1.0
 */
public class PostionMatrixServiceImpl extends Service implements PostionMatrixService {

    @SneakyThrows
    @Override
    public PostionMatrixVo postionMatrix() {


        RecordSet rs = new RecordSet();
        //1.人员信息获取
        List<ResourcesPo> resourcesPoList = postionPeoples();

        //2.报表岗位层级数据
        LinkedList<Postion> tops = new LinkedList<>();
        rs.executeQuery("select a.id,a.ywzwxl,a.ywzwcj,b.ywzwcjywzd from uf_ywzwcj a left join uf_ywjsglgx b on a.id = b.ywzwcj where a.ywzwxl = 2");
        while (rs.next()) {
            tops.add(Postion.builder()
                    .id(Util.null2String(rs.getString("id"))+","+Util.null2String(rs.getString("ywzwcjywzd")))
                    .postionType(Util.getIntValue(rs.getString("ywzwxl")))
                    .postionName(Util.null2String(rs.getString("ywzwcj")).replaceAll("(技术)", "$1（业务）"))
                    .build());
        }


        //3.双层循环构造中间数据
        LinkedList<PostionPeople> centers = new LinkedList<>();
        DepartmentComInfo dept = new DepartmentComInfo();
        Map<String, List<ResourcesPo>> groupedByDepartment = resourcesPoList.stream()
                .collect(Collectors.groupingBy(ResourcesPo::getDepartment));
        for (Map.Entry<String, List<ResourcesPo>> entry : groupedByDepartment.entrySet()) {
            String department = entry.getKey();
            List<ResourcesPo> poList = entry.getValue();

            //职能管理序列
            List<ResourcesPo> fDatasAll = poList.stream()
                    .filter(po -> ("1".equals(po.getPostionType()) && department.equals(po.getDepartment())))
                    .collect(Collectors.toList());

            LinkedList<PeoplesAndNames> fDatas = new LinkedList<>();
            if (fDatasAll.size() > 0) {
                for (int i = 0; i < tops.size(); i++) {
                    Postion postion = tops.get(i);
                    String[] parts = postion.getId().trim().split("\\s*,\\s*");
                    List<ResourcesPo> collect = fDatasAll.stream()
                            .filter(po -> parts[1].equals(po.getPostion()))
                            .collect(Collectors.toList());
                    String names = collect.stream()
                            .map(ResourcesPo::getResource)
                            .collect(Collectors.joining("、"));

                    fDatas.add(PeoplesAndNames.builder()
                            .peoples(collect.size())
                            .names(names)
                            .build());
                }
            }

            //专业技术序列
            List<ResourcesPo> sDatasAll = poList.stream()
                    .filter(po -> ("2".equals(po.getPostionType()) && department.equals(po.getDepartment())))
                    .collect(Collectors.toList());
            LinkedList<PeoplesAndNames> sDatas = new LinkedList<>();
            if (sDatasAll.size() > 0) {
                for (int i = 0; i < tops.size(); i++) {
                    Postion postion = tops.get(i);
                    String[] parts = postion.getId().trim().split("\\s*,\\s*");
                    List<ResourcesPo> collect = sDatasAll.stream()
                            .filter(po -> parts[0].equals(po.getPostion()))
                            .collect(Collectors.toList());
                    String names = collect.stream()
                            .map(ResourcesPo::getResource)
                            .collect(Collectors.joining("、"));

                    sDatas.add(PeoplesAndNames.builder()
                            .peoples(collect.size())
                            .names(names)
                            .build());
                }
            }

            int row = 1;
            if (fDatasAll.size() > 0 && sDatasAll.size() > 0) {
                row = 2;
            }

            centers.add(PostionPeople.builder()
                    .name(dept.getDepartmentName(department))
                    .row(row)
                    .fsum(fDatasAll.size())
                    .ssum(sDatasAll.size())
                    .fdatas(fDatas)
                    .sdatas(sDatas)
                    .build());
        }

        //4.底部合计数据
        LinkedList<PeopleSums> footers = new LinkedList<>();
        //1.职能管理序列
        int znSums = 0;
        LinkedList<Integer> zndatas = new LinkedList<>();
        for (int i = 0; i < tops.size(); i++) {
            Postion postion = tops.get(i);
            String[] parts = postion.getId().trim().split("\\s*,\\s*");
            int size = (int) resourcesPoList.stream()
                    .filter(res ->
                            Objects.equals(res.getPostion(), parts[1]) &&
                                    Objects.equals(res.getPostionType(), "1")
                    )
                    .count();

            zndatas.add(size);
            //名单列 增加null代替不展示
            zndatas.add(null);
            if (i == tops.size() - 1) {
                rs.executeQuery("select count(DISTINCT xm) as sums from uf_ywzwxx where ywzwxl = 1 and rbywzwcjqssj is not null");
                if (rs.next()) {
                    znSums = Util.getIntValue(rs.getString("sums"),0);
                }
                zndatas.add(znSums);
            }

        }
        footers.add(PeopleSums.builder().type("职能管理序列").datas(zndatas).build());



        //3.总计
        LinkedList<Integer> zjdatas = new LinkedList<>();
        int zjSums = 0;
        for (int i = 0; i < tops.size(); i++) {
            Postion postion = tops.get(i);
            String[] parts = postion.getId().trim().split("\\s*,\\s*");
            int size = (int) resourcesPoList.stream()
                    .filter(res ->
                            Objects.equals(res.getPostion(), parts[0]) ||
                                    Objects.equals(res.getPostion(), parts[1])
                    )
                    .count();
            zjdatas.add(size);
            //名单列 增加null代替不展示
            zjdatas.add(null);

            if (i == tops.size() - 1) {
                rs.executeQuery("select count(DISTINCT xm) as sums from uf_ywzwxx where rbywzwcjqssj is not null");
                if (rs.next()) {
                    zjSums = Util.getIntValue(rs.getString("sums"),0);
                }
                zjdatas.add(zjSums);
            }
        }


        //2.专业技术序列
        LinkedList<Integer> zydatas = new LinkedList<>();
        for (int i = 0; i < tops.size(); i++) {
            Postion postion = tops.get(i);
            String[] parts = postion.getId().trim().split("\\s*,\\s*");
            int size = (int) resourcesPoList.stream()
                    // 空安全比较（支持字段值为null的情况）
                    .filter(res ->
                            Objects.equals(res.getPostion(), parts[0]) &&
                                    Objects.equals(res.getPostionType(), "2")
                    )
                    .count();
            zydatas.add(size);
            //名单列 增加null代替不展示
            zydatas.add(null);
            if (i == tops.size() - 1) {
                zydatas.add(zjSums - znSums);
            }

        }
        footers.add(PeopleSums.builder().type("专业技术序列").datas(zydatas).build());
        footers.add(PeopleSums.builder().type("总计").datas(zjdatas).build());

        return PostionMatrixVo.builder().tops(tops).centers(centers).footers(footers).build();
    }

    @Override
    @SneakyThrows
    public List<ResourcesPo> postionPeoples() {
        RecordSet rs = new RecordSet();
        List<ResourcesPo> resourcesList = new ArrayList<>();
        ResourceComInfo resource = new ResourceComInfo();
        rs.executeQuery("select id,xm,szbm,ywzwxl,ywzwcj,rbywzwcjqssj from uf_ywzwxx where rbywzwcjqssj is not null");
        while (rs.next()) {
            resourcesList.add(ResourcesPo.builder()
                    .id(Util.getIntValue(rs.getString("id")))
                    .resourceId(Util.null2String(rs.getString("xm")))
                    .resource(resource.getLastname(Util.null2String(rs.getString("xm"))))
                    .department(Util.null2String(rs.getString("szbm")))
                    .postionType(Util.null2String(rs.getString("ywzwxl")))
                    .postion(Util.null2String(rs.getString("ywzwcj")))
                    .workDate(Util.null2String(rs.getString("rbywzwcjqssj")))
                    .build());
        }

        //过滤后数据
        return filterClosestWorkDate(resourcesList);
    }

    @Override
    public  List<ResourcesPo> filterClosestWorkDate(List<ResourcesPo> resourcesPos) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 按 resourceId 分组
        Map<String, List<ResourcesPo>> groupedByResource = resourcesPos.stream()
                .collect(Collectors.groupingBy(ResourcesPo::getResourceId));

        List<ResourcesPo> result = new ArrayList<>();
        // 遍历分组结果
        groupedByResource.forEach((resource, group) -> {
            try {
                // 找出该分组中 workDate 最大（最近）的元素
                Optional<ResourcesPo> maxDatePo = group.stream()
                        .max(Comparator.comparing((ResourcesPo po) -> {
                            try {
                                return sdf.parse(po.getWorkDate());
                            } catch (ParseException e) {
                                throw new RuntimeException(e);
                            }
                        }).thenComparingInt(ResourcesPo::getId));
                maxDatePo.ifPresent(result::add);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        return result;

    }
}
