package weaver.interfaces.jclproduct;


import cn.hutool.core.collection.CollectionUtil;
import com.engine.jclproduct.entity.bo.EmploymentRecordBo;
import com.engine.jclproduct.entity.po.DataDutyPo;
import com.engine.jclproduct.entity.po.EmploymentRecordPo;
import com.engine.jclproduct.entity.po.HrmresourcePo;
import com.engine.jclproduct.utils.EmploymentUtil;
import com.weaver.file.ConfigOperator;
import com.weaver.general.BaseBean;
import com.weaver.general.TimeUtil;
import com.weaver.general.Util;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @Author liang.cheng
 * @Date 2024/10/22 4:24 PM
 * @Description: 人员入转调离 任职变化 记录任务
 * @Version 1.0
 */

public class EmploymentRecordCrob extends BaseCronJob {

    @Override
    public void execute() {

        RecordSet rs = new RecordSet();
        //1.获取当前人员信息数据
        List<HrmresourcePo> hrmresourcePoList = EmploymentUtil.getHrmResources();

        //人员信息表更新集合
        List<HrmresourcePo> hrmresourceUpdate = new ArrayList<>();

        //人员信息表新增集合
        List<HrmresourcePo> hrmresourceInsert = new ArrayList<>();

        //任职记录表新增集合
        List<EmploymentRecordPo> employmentRecordInsert = new ArrayList<>();

        BaseBean bb = new BaseBean();
        String formModeId = bb.getPropValue("jclproduct", "eModeId");

        String modeDataCreateDate = DateUtil.getCurrentDate();
        String modeDataCreateTime = TimeUtil.getOnlyCurrentTimeString();
        ConfigOperator configOperator = new ConfigOperator();
        configOperator.setProp("jclproduct.properties", "startDate", modeDataCreateDate);

        //2.与人力资源中间表对比每一条人员数据 todo 数据量大的情况下线程处理
        hrmresourcePoList.forEach(hrmresource->{

            //1.入转调离 人员数据变化处理
            DataDutyPo dataDutyPo = dutyDatasChange(hrmresource);
            if (dataDutyPo.getHrmresourceUpdate() != null) {
                hrmresourceUpdate.add(dataDutyPo.getHrmresourceUpdate());
            }
            if (dataDutyPo.getHrmresourceInsert() != null) {
                hrmresourceInsert.add(dataDutyPo.getHrmresourceInsert());
            }

            if (CollectionUtil.isNotEmpty(dataDutyPo.getEmploymentRecordInsert())) {
                employmentRecordInsert.addAll(dataDutyPo.getEmploymentRecordInsert());
            }

        });

        bb.writeLog("hrmresourceUpdate:"+hrmresourceUpdate.size());
        bb.writeLog("hrmresourceInsert:"+hrmresourceInsert.size());
        bb.writeLog("employmentRecordInsert:"+employmentRecordInsert.size());
        //3.人员信息中间表
        //更新
        hrmresourceUpdate.forEach(e -> rs.executeUpdate("update uf_hrmresource_day set status=?,companystartdate=?,companyworkyear=?," +
                " sex=?,birthday=?,jobtitle=?,departmentid=?,subcompanyid=?,managerid=? where userid=?",e.getStatus(),e.getCompanyStartDate(),
                e.getCompanyWorkyear(),e.getSex(),e.getBirthday(),e.getJobTitle(),e.getDepartmentId(),e.getSubcompanyId(),e.getManagerId(),e.getUserId()));

        //插入
        hrmresourceInsert.forEach(e-> rs.executeUpdate("insert into uf_hrmresource_day(userId,workcode,companystartdate,companyworkyear," +
                        " birthday,jobtitle,departmentid,subcompanyid,managerid,status,sex) values(?,?,?,?,?,?,?,?,?,?,?)",e.getUserId(),e.getWorkcode(),
                e.getCompanyStartDate(),e.getCompanyWorkyear(),e.getBirthday(),e.getJobTitle(),e.getDepartmentId(),e.getSubcompanyId(),e.getManagerId(),e.getStatus(),e.getSex()));


        //4.人员任职记录表
        employmentRecordInsert.forEach(e->{

            //1.更新上一条数据stopDate
            List<EmploymentRecordPo> list = new ArrayList<>();
            rs.executeQuery("select id,startDate from uf_EmploymentRecord where userid = ?",e.getUserId());
            while (rs.next()) {
                list.add(EmploymentRecordPo.builder().id(Util.getIntValue(rs.getString("id"))).startDate(Util.null2String(rs.getString("startDate"))).build());
            }

            if (CollectionUtil.isNotEmpty(list)) {
                String date = DateUtil.getCurrentDate();
                EmploymentRecordPo latest = list.stream()
                        .max(Comparator.comparing(EmploymentRecordPo::getId))
                        .orElse(null);
                assert latest != null;
                if (EmploymentUtil.compareDate(latest.getStartDate())) {
                    date = DateUtil.getYesterday();
                }

                rs.executeUpdate("update uf_EmploymentRecord set stopDate = ? where id = ?",date,latest.getId());
            }


            //2.新增数据
            rs.executeUpdate("insert into uf_EmploymentRecord(userId,workcode,employeeStatus,changType,startDate,stopDate," +
                            " departmentId,subCompanyId,superior,position,changeOldSuperior,changeNewSuperior,changeOldJob,changeNewJob,changeOldDept,changeNewDept," +
                            " companyStartDate,workyear,entryDateYear,entryDateMonth,entryDateDay,expectedLastWorkDate,lastWorkDate,probationActualStopDate,regularizationDate," +
                            " sex,birthday,formmodeid,modeDataCreateDate,modeDataCreateTime,modeDataCreater) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
                    e.getUserId(),e.getWorkcode(),e.getEmployeeStatus(),e.getChangType(),e.getStartDate(),e.getStopDate(),e.getDepartmentId(),
                    e.getSubCompanyId(),e.getSuperior(),e.getPosition(),e.getChangeOldSuperior(),e.getChangeNewSuperior(),
                    e.getChangeOldJob(),e.getChangeNewJob(),e.getChangeOldDept(),e.getChangeNewDept(),e.getCompanyStartDate(),
                    e.getWorkyear(),e.getEntryDateYear(),e.getEntryDateMonth(),e.getEntryDateDay(), e.getExpectedLastWorkDate(),
                    e.getLastWorkDate(),e.getProbationActualStopDate(),e.getRegularizationDate(),e.getSex(),e.getBirthday(),formModeId,modeDataCreateDate,modeDataCreateTime,1);
        });
    }

    /**
     * 入转调离 人员数据变化处理
     * @param hrmresource
     */
    private DataDutyPo dutyDatasChange(HrmresourcePo hrmresource){
        DataDutyPo dataDutyPo = new DataDutyPo();
        RecordSet rs = new RecordSet();
        //2.1 根据userid获取上一次人员中间表数据作对比
        rs.executeQuery("select id,userid,workcode,status,companystartdate,companyworkyear,sex,birthday," +
                " jobtitle,departmentid,subcompanyid,managerid  from uf_hrmresource_day where userid = ?",hrmresource.getUserId());

        if (rs.next()) {
            //3.数据存在作比较
            HrmresourcePo lastResource = HrmresourcePo.builder()
                    .id(Util.getIntValue(rs.getString("id")))
                    .status(Util.getIntValue(rs.getString("status")))
                    .companyWorkyear(Util.null2String(rs.getString("companyworkyear")))
                    .subcompanyId(Util.getIntValue(rs.getString("subcompanyid")))
                    .departmentId(Util.getIntValue(rs.getString("departmentid")))
                    .jobTitle(Util.getIntValue(rs.getString("jobtitle")))
                    .managerId(Util.getIntValue(rs.getString("managerid")))
                    .build();

            //3.0 数据一致不做处理
            //3.11 人员状态比较
            List<EmploymentRecordPo> employmentRecords = new ArrayList<>();
            if (!hrmresource.getStatus().equals(lastResource.getStatus())) {
                //状态变更
                EmploymentRecordPo employmentRecord = EmploymentRecordBo.setEmploymentRecordPo(hrmresource);
                //记录各种状态下日期
                EmploymentUtil.dutyStatusDate(employmentRecord);
                employmentRecords.add(employmentRecord);

            }

            //3.12 调动状态比较（分别是 部门 岗位 人员上级 任一个条件发生变化 都记录为调动）
            if (!hrmresource.getDepartmentId().equals(lastResource.getDepartmentId())
                    || !hrmresource.getJobTitle().equals(lastResource.getJobTitle())
                    || !hrmresource.getManagerId().equals(lastResource.getManagerId())){

                EmploymentRecordPo employmentRecord = EmploymentRecordBo.setEmploymentRecordPo(hrmresource);
                // 记录调动下新旧组织数据留痕
                employmentRecord.setChangeOldDept(lastResource.getDepartmentId());
                employmentRecord.setChangeOldJob(lastResource.getJobTitle());
                employmentRecord.setChangeOldSuperior(lastResource.getManagerId());

                employmentRecord.setChangeNewDept(hrmresource.getDepartmentId());
                employmentRecord.setChangeNewJob(hrmresource.getJobTitle());
                employmentRecord.setChangeNewSuperior(hrmresource.getManagerId());

                //调动
                employmentRecord.setChangType(2);
                employmentRecords.add(employmentRecord);
            }


            if (!hrmresource.getStatus().equals(lastResource.getStatus())
                    || !hrmresource.getDepartmentId().equals(lastResource.getDepartmentId())
                    || !hrmresource.getJobTitle().equals(lastResource.getJobTitle())
                    || !hrmresource.getManagerId().equals(lastResource.getManagerId())) {
                //3.1 数据不一致，人员中间表更新
                hrmresource.setId(lastResource.getId());
                dataDutyPo.setHrmresourceUpdate(hrmresource);
                //3.2 数据不一致，任职记录表新增
                dataDutyPo.setEmploymentRecordInsert(employmentRecords);
            }

        }else {
            //4.数据不存在作新增
            //4.1 人员中间表新增
            dataDutyPo.setHrmresourceInsert(hrmresource);
            //4.2 任职记录表新增
            List<EmploymentRecordPo> recordList = Collections.singletonList(EmploymentRecordBo.setEmploymentRecordPo(hrmresource));
            dataDutyPo.setEmploymentRecordInsert(recordList);
        }

        return dataDutyPo;
    }


}
