package weaver.interfaces.jclproduct;

import cn.hutool.core.collection.CollectionUtil;
import com.engine.jclproduct.entity.bo.EmploymentRecordBo;
import com.engine.jclproduct.entity.po.DepartmentPo;
import com.engine.jclproduct.entity.po.OrganizationRecordPo;
import com.engine.jclproduct.entity.po.SubCompanyPo;
import com.engine.jclproduct.utils.EmploymentUtil;
import com.weaver.general.Util;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.interfaces.schedule.BaseCronJob;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * @Author liang.cheng
 * @Date 2024/11/11 10:25 AM
 * @Description: 组织启用 封存 变动记录（上级变动 名称变动 负责人变动）
 * @Version 1.0
 */
public class OrganizationRecordCrob extends BaseCronJob {

    @Override
    public void execute() {

        RecordSet rs = new RecordSet();
        //1.获取当前分部信息数据
        List<SubCompanyPo> subCompany = EmploymentUtil.getSubCompany();

        //2.获取当前部门信息数据
        List<DepartmentPo> department = EmploymentUtil.getDepartment();

        //3.分部中间表更新集合
        List<SubCompanyPo> subCompanyUpdate = new ArrayList<>();

        //4.分部中间表新增集合
        List<SubCompanyPo> subCompanyInsert = new ArrayList<>();

        //5.部门中间表更新集合
        List<DepartmentPo> departmentUpdate = new ArrayList<>();

        //6.部门中间表新增集合
        List<DepartmentPo> departmentInsert = new ArrayList<>();

        //7.组织变动表新增集合
        List<OrganizationRecordPo> organizationRecordInsert = new ArrayList<>();


        //8.分部数据对比
        subCompany.forEach(e -> dutySubChange(e,subCompanyUpdate,subCompanyInsert,organizationRecordInsert));

        //9.部门数据对比
        department.forEach(e-> dutyDeptChange(e,departmentUpdate,departmentInsert,organizationRecordInsert));

        //10.数据更新(todo 是否考虑事务一致性)
        subCompanyUpdate.forEach(e-> rs.executeUpdate("update uf_hrmsubcompany set subcompanyName=?,supsubcomId=?,showorder=?,canceled=?,subcompanyCode=? where subcompanyId = ?",e.getSubcompanyName(),
                e.getSupsubcomId(),e.getShoworder(),e.getCanceled(),e.getSubcompanyCode(),e.getSubcompanyId()));

        subCompanyInsert.forEach(e-> rs.executeUpdate("insert into uf_hrmsubcompany(subcompanyId,subcompanyName,supsubcomId,showorder,canceled,subcompanyCode) values(?,?,?,?,?,?)",e.getSubcompanyId(),e.getSubcompanyName(),
                e.getSupsubcomId(),e.getShoworder(),e.getCanceled(),e.getSubcompanyCode()));

        departmentUpdate.forEach(e-> rs.executeUpdate("update uf_hrmdepartment set departmentmark=?,subcompanyId=?,supdepId=?,showorder=?,canceled=?,departmentCode=?,bmfzr=? where departmentId = ?",e.getDepartmentmark(),e.getSubcompanyId(),
                e.getSupdepId(),e.getShoworder(),e.getCanceled(),e.getDepartmentCode(),e.getBmfzr(),e.getDepartmentId()));

        departmentInsert.forEach(e -> rs.executeUpdate("insert into uf_hrmdepartment(departmentId,departmentmark,subcompanyId,supdepId,showorder,canceled,departmentCode,bmfzr) values(?,?,?,?,?,?,?,?)",e.getDepartmentId(),e.getDepartmentmark(),e.getSubcompanyId(),
                e.getSupdepId(),e.getShoworder(),e.getCanceled(),e.getDepartmentCode(),e.getBmfzr()));

        organizationRecordInsert.forEach(e -> {
            //1.更新上一条数据stopDate
            List<OrganizationRecordPo> list = new ArrayList<>();
            rs.executeQuery("select id,startDate from uf_orgChange where OId = ? and oType = ?",e.getOId(),e.getOType());
            while (rs.next()) {
                list.add(OrganizationRecordPo.builder().id(Util.getIntValue(rs.getString("id"))).startDate(Util.null2String(rs.getString("startDate"))).build());
            }

            if (CollectionUtil.isNotEmpty(list)) {
                String date = DateUtil.getCurrentDate();
                OrganizationRecordPo latest = list.stream()
                        .max(Comparator.comparing(OrganizationRecordPo::getId))
                        .orElse(null);
                assert latest != null;
                if (EmploymentUtil.compareDate(latest.getStartDate())) {
                    date = DateUtil.getYesterday();
                }
                rs.executeUpdate("update uf_orgChange set stopDate = ? where id = ?",date,latest.getId());
            }

            rs.executeUpdate("insert into uf_orgChange(orgStatus,startDate,stopDate,oId,oType,oName,oParentId,oChargeId,changeType,showorder,organizationCode,newName,newParentId,newOChargeId,oldName,oldParentId,oldOChargeId,belongSub) " +
                            " values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",e.getOrgStatus(), e.getStartDate(),e.getStopDate(),e.getOId(),e.getOType(),e.getOName(),e.getOParentId(),e.getOChargeId(),e.getChangeType(),e.getShoworder(),e.getOrganizationCode(),e.getNewName(),
                    e.getNewParentId(),e.getNewOChargeId(),e.getOldName(),e.getOldParentId(),e.getOldOChargeId(),e.getBelongSub());
        });

    }

    /**
     * 部门数据变化
     * @param departmentPo
     * @param departmentUpdate
     * @param departmentInsert
     * @param organizationRecordInsert
     */
    private void dutyDeptChange(DepartmentPo departmentPo, List<DepartmentPo> departmentUpdate, List<DepartmentPo> departmentInsert, List<OrganizationRecordPo> organizationRecordInsert) {
        RecordSet rs = new RecordSet();
        //2.1 根据departmentId获取上一次部门中间表数据作对比
        rs.executeQuery("select departmentId,departmentmark,subcompanyId,supdepId,showorder,canceled,departmentCode,bmfzr from uf_hrmdepartment where departmentId = ?",departmentPo.getDepartmentId());
        if (rs.next()) {
            DepartmentPo lastDepartmentPo = DepartmentPo.builder()
                    .departmentId(Util.getIntValue(rs.getString("departmentId")))
                    .departmentmark(Util.null2String(rs.getString("departmentmark")))
                    .subcompanyId(Util.getIntValue(rs.getString("subcompanyId")))
                    .supdepId(Util.getIntValue(rs.getString("supdepId")))
                    .showorder(Util.getFloatValue(rs.getString("showorder")))
                    .canceled(Util.getIntValue(rs.getString("canceled")))
                    .departmentCode(Util.null2String(rs.getString("departmentCode")))
                    .bmfzr(Util.null2String(rs.getString("bmfzr")))
                    .build();
            //1.状态比较
            if (!departmentPo.getCanceled().equals(lastDepartmentPo.getCanceled())) {
                departmentUpdate.add(departmentPo);
                OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(null, departmentPo, 1);
                organizationRecordInsert.add(organizationRecordPo);
            }
            //2.变动比较
            if (!departmentPo.getSupdepId().equals(lastDepartmentPo.getSupdepId())
                    || !departmentPo.getDepartmentmark().equals(lastDepartmentPo.getDepartmentmark())
            || !departmentPo.getBmfzr().equals(lastDepartmentPo.getBmfzr())) {
                departmentUpdate.add(departmentPo);
                // 记录调动下新旧组织数据留痕
                OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(null, departmentPo, 1);
                organizationRecordPo.setChangeType(1);
                organizationRecordPo.setOldParentId(lastDepartmentPo.getSupdepId());
                organizationRecordPo.setOldName(lastDepartmentPo.getDepartmentmark());
                organizationRecordPo.setOldOChargeId(lastDepartmentPo.getBmfzr());
                organizationRecordPo.setNewParentId(departmentPo.getSupdepId());
                organizationRecordPo.setNewName(departmentPo.getDepartmentmark());
                organizationRecordPo.setNewOChargeId(departmentPo.getBmfzr());
                organizationRecordInsert.add(organizationRecordPo);
            }
        }else {
            departmentInsert.add(departmentPo);
            OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(null, departmentPo, 1);
            organizationRecordInsert.add(organizationRecordPo);
        }

    }

    /**
     * 分部数据变化
     * @param subCompanyPo
     * @param subCompanyUpdate
     * @param subCompanyInsert
     */
    private void dutySubChange(SubCompanyPo subCompanyPo, List<SubCompanyPo> subCompanyUpdate, List<SubCompanyPo> subCompanyInsert,List<OrganizationRecordPo> organizationRecordInsert) {

        RecordSet rs = new RecordSet();
        //2.1 根据subcompanyId获取上一次分部中间表数据作对比
        rs.executeQuery("select subcompanyId,subcompanyName,supsubcomId,showorder,canceled,subcompanyCode from uf_hrmsubcompany where subcompanyId = ?",subCompanyPo.getSubcompanyId());
        if (rs.next()) {
            //数据存在作比较
            SubCompanyPo lastSubCompany = SubCompanyPo.builder()
                    .subcompanyId(Util.getIntValue(rs.getString("subcompanyId")))
                    .subcompanyName(Util.null2String(rs.getString("subcompanyname")))
                    .supsubcomId(Util.getIntValue(rs.getString("supsubcomId")))
                    .showorder(Util.getFloatValue(rs.getString("showorder")))
                    .canceled(Util.getIntValue(rs.getString("canceled")))
                    .subcompanyCode(Util.null2String(rs.getString("subcompanycode")))
                    .build();

            // 状态比较 （启用 封存 E9物理删除无比较意义）
            if (!subCompanyPo.getCanceled().equals(lastSubCompany.getCanceled())) {
                subCompanyUpdate.add(subCompanyPo);
                OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(subCompanyPo, null, 0);
                organizationRecordInsert.add(organizationRecordPo);
            }

            //变动比较
            if (!subCompanyPo.getSupsubcomId().equals(lastSubCompany.getSupsubcomId())
                    || !subCompanyPo.getSubcompanyName().equals(lastSubCompany.getSubcompanyName())) {
                subCompanyUpdate.add(subCompanyPo);
                // 记录调动下新旧组织数据留痕
                OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(subCompanyPo, null, 0);
                organizationRecordPo.setChangeType(1);
                organizationRecordPo.setOldParentId(lastSubCompany.getSupsubcomId());
                organizationRecordPo.setOldName(lastSubCompany.getSubcompanyName());
                organizationRecordPo.setNewParentId(subCompanyPo.getSupsubcomId());
                organizationRecordPo.setNewName(subCompanyPo.getSubcompanyName());
                organizationRecordInsert.add(organizationRecordPo);
            }

        }else {
            //数据不存在新增
            subCompanyInsert.add(subCompanyPo);
            OrganizationRecordPo organizationRecordPo = EmploymentRecordBo.setOrganizationRecordPo(subCompanyPo, null, 0);
            organizationRecordInsert.add(organizationRecordPo);
        }


    }
}
